using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using System.Reflection;
using System.ComponentModel;

namespace ASPItalia.Books.Web.Chapter11
{


    public abstract class EntityManager<T>
        where T: new()
    {

        private static List<T> cache = new List<T>();

        public T CreateNewEntity()
        {
            return new T();
        }

        protected abstract T FindEntity(List<T> cache, T entity);

        public virtual List<T> ListAll()
        {
            // Restituisco una copia
            return new List<T>(cache);
        }

        public virtual List<T> ListAll(int maximumRows, int startRowIndex)
        {
            // Se  uguale a zero o minore sono richieste le intere righe
            if (maximumRows <= 0) return this.ListAll();

            List<T> list = new List<T>(maximumRows);
            // Restituisco una copia in base all'indice di partenza
            for (int i = startRowIndex; i < cache.Count && i < startRowIndex + maximumRows; i++)
                list.Add(cache[i]);

            list.TrimExcess();
            return list;
        }

        public virtual int GetTotalEntities()
        {
            return cache.Count;
        }

        public virtual T GetSingle(T entity)
        {
            return this.FindEntity(cache, entity);
        }

        public T Add(T entity)
        {
            // Controllo che non esista gi
            if (this.FindEntity(cache, entity) != null)
                throw new ArgumentException("Entity already exists");

            cache.Add(entity);

            return entity;
        }

        public virtual T Update(T entity, T old_entity)
        {
            T original = this.FindEntity(cache, entity);
            if (original == null)
                throw new ArgumentException("Cannot find entity specified");

            // Controllo la presenza di conflitti se richiesto
            if (old_entity != null)
            {
                CheckConflict(old_entity, original);
            }

            // Sostituisco quella vecchia con quella nuova
            cache.Remove(original);
            cache.Add(entity);

            return original;            
        }

        private void CheckConflict(T old_entity, T original)
        {
            // Ottengo la lista delle propriet
            PropertyDescriptorCollection piList = TypeDescriptor.GetProperties(typeof(T));
            foreach (PropertyDescriptor pd in piList)
            {
                object v = pd.GetValue(original);
                object v2 = pd.GetValue(old_entity);
                // Se il valore origiane non corrisponde a quello precedente
                // nel frattempo  stato cambiato
                if (v2 != null && v != v2 && !v.Equals(v2))
                    throw new HttpException("The entity was changed during edit");
            }
        }

        public virtual T Update(T entity)
        {
            return this.Update(entity, default(T));
        }

        public virtual void Delete(T entity)
        {
            T original = this.FindEntity(cache, entity);
            if (original == null)
                throw new ArgumentException("Cannot find entity specified");

            cache.Remove(original);
        }
    }
}