using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections;
using System.IO;

namespace ASPItalia.Books.Web.Chapter11
{

    /// <summary>
    /// DataSource che permette l'accesso ai files e alle directory
    /// </summary>
    public class FileSystemDataSource : HierarchicalDataSourceControl, IDataSource
    {

        private string[] _viewNames;

        protected override HierarchicalDataSourceView GetHierarchicalView(string viewPath)
        {
            return new FileSystemHierarchicalDataSourceView(this, viewPath);
        }

        /// <summary>
        /// Percorso assoluto o relativo della directory base dalla quale sfogliare
        /// </summary>
        public string BaseDirectory
        {
            get
            {
                object o = ViewState["BaseDirectory"];
                if (o != null)
                    return (string)o;
                return String.Empty;
            }
            set
            {
                ViewState["BaseDirectory"] = value;

                this.OnDataSourceChanged(EventArgs.Empty);
            }
        }

        /// <summary>
        /// Indica se mostrare in modo gerarchico anche i files. Default false
        /// </summary>
        public bool ShowHierarchicalFiles
        {
            get
            {
                object o = ViewState["ShowHierarchicalFiles"];
                if (o != null)
                    return (bool)o;
                else
                    return false;
            }
            set
            {
                ViewState["ShowHierarchicalFiles"] = true;

                this.OnDataSourceChanged(EventArgs.Empty);
            }
        }

        internal DirectoryInfo BaseDirectoryInfo
        {
            get
            {
                string path = this.BaseDirectory;
                if (!Path.IsPathRooted(path))
                    path = this.Context.Request.MapPath(path);
                return new DirectoryInfo(path);
            }
        }


        #region IDataSource Members

        DataSourceView IDataSource.GetView(string viewName)
        {
            if (viewName.Length == 0)
                viewName = "DefaultView";

            return new FileSystemDataSourceView(this, viewName);
        }

        ICollection IDataSource.GetViewNames()
        {
            if (this._viewNames == null)
                this._viewNames = new string[] { "DefaultView" };

            return this._viewNames;
        }

        event EventHandler IDataSource.DataSourceChanged
        {
            add { ((IHierarchicalDataSource)this).DataSourceChanged += value; }
            remove { ((IHierarchicalDataSource)this).DataSourceChanged -= value; }
        }

        #endregion
    }

}