using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;


namespace ASPItalia.Books.Web.Chapter11
{

    /// <summary>
    /// Vista flat rappresentante una view
    /// </summary>
    public class FileSystemDataSourceView : DataSourceView
    {

        private FileSystemDataSource _owner;

        internal FileSystemDataSourceView(FileSystemDataSource owner, string viewName) : base(owner, viewName)
        {
            this._owner = owner;
        }

        protected override IEnumerable ExecuteSelect(DataSourceSelectArguments arguments)
        {
            arguments.RaiseUnsupportedCapabilitiesError(this);

            List<FileSystemItem> list = new List<FileSystemItem>();

            // Aggiungo le directory
            foreach (FileSystemInfo di in this._owner.BaseDirectoryInfo.GetDirectories())
                list.Add(new FileSystemItem(di));

            // Aggiungo i files
            foreach (FileSystemInfo fi in this._owner.BaseDirectoryInfo.GetFiles())
                list.Add(new FileSystemItem(fi));

            return list;
        }
    }

    public class FileSystemItem
    {

        private FileSystemInfo file;

        internal FileSystemItem(FileSystemInfo file)
        {
            this.file = file;
        }

        public FileAttributes Attributes
        {
            get { return this.file.Attributes; }
        }

        public DateTime CreationTime
        {
            get { return this.file.CreationTime; }
        }

        public long Size
        {
            get
            {
                if (this.file is FileInfo)
                    return ((FileInfo)file).Length;
                else
                    return 0;
            }
        }

        public string Extension
        {
            get { return this.file.Extension; }
        }

        public virtual string FullName
        {
            get { return this.file.FullName; }
        }

        public DateTime LastAccessTime
        {
            get { return this.file.LastAccessTime; }
        }

        public DateTime LastWriteTime
        {
            get { return this.file.LastAccessTime; }
        }

        public string Name
        {
            get { return this.file.Name; }
        }

        //#region ICustomTypeDescriptor Members

        //System.ComponentModel.AttributeCollection ICustomTypeDescriptor.GetAttributes()
        //{
        //    return null;
        //}

        //string ICustomTypeDescriptor.GetClassName()
        //{
        //    return this.GetType().Name;
        //}

        //string ICustomTypeDescriptor.GetComponentName()
        //{
        //    return null;
        //}

        //TypeConverter ICustomTypeDescriptor.GetConverter()
        //{
        //    return null;
        //}

        //EventDescriptor ICustomTypeDescriptor.GetDefaultEvent()
        //{
        //    return null;
        //}

        //PropertyDescriptor ICustomTypeDescriptor.GetDefaultProperty()
        //{
        //    return null;
        //}

        //object ICustomTypeDescriptor.GetEditor(Type editorBaseType)
        //{
        //    return null;
        //}

        //EventDescriptorCollection ICustomTypeDescriptor.GetEvents(Attribute[] attributes)
        //{
        //    return null;
        //}

        //EventDescriptorCollection ICustomTypeDescriptor.GetEvents()
        //{
        //    return null;
        //}

        //PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties(Attribute[] attributes)
        //{
        //    PropertyDescriptorCollection list = new PropertyDescriptorCollection(
        //}

        //PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties()
        //{
        //    return ((ICustomTypeDescriptor)this).GetProperties(null);
        //}

        //object ICustomTypeDescriptor.GetPropertyOwner(PropertyDescriptor pd)
        //{
        //    return this;
        //}

        //#endregion
    }

}