using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections;
using System.IO;

namespace ASPItalia.Books.Web.Chapter11
{
    /// <summary>
    /// Vista per il FileSystemDataSource
    /// </summary>
    public class FileSystemHierarchicalDataSourceView : HierarchicalDataSourceView
    {

        private FileSystemDataSource owner;
        private string viewPath;

        internal FileSystemHierarchicalDataSourceView(FileSystemDataSource owner, string viewPath)
        {
            this.owner = owner;
            this.viewPath = viewPath;
        }
 
        public override IHierarchicalEnumerable Select()
        {
            // Controllo la validit della directory
            if (this.owner.BaseDirectory.Length == 0)
                throw new HttpException(String.Format("Invalid BaseDirectory for control {0}", this.owner.ID));

            DirectoryInfo path = this.owner.BaseDirectoryInfo;
            DirectoryInfo parent = null;

            if (!String.IsNullOrEmpty(viewPath))
            {
                string[] steps = viewPath.Split(new char[] { '/' }, StringSplitOptions.RemoveEmptyEntries);
                // Entro ricorsivamente all'interno della directory
                foreach (string step in steps)
                {
                    // Memorizzo il precedente path come padre
                    parent = path;
                    // Cerco la sottodirectory
                    DirectoryInfo[] childDirs = path.GetDirectories(step, SearchOption.TopDirectoryOnly);
                    if (childDirs.Length == 0)
                        throw new HttpException(String.Format("Child directory {0} in {1} does not exist", step, path.FullName));

                    path = childDirs[0];
                }
            }

            // Restituisco l'enumeratore per quella directory
            return new FileSystemHierarchicalEnumerable(path, null, viewPath, this);
        }

        internal bool ShowHierarchicalFiles
        {
            get { return this.owner.ShowHierarchicalFiles; }
        }
    }

}