using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Collections;

namespace ASPItalia.Books.Web.Chapter11
{

    /// <summary>
    ///  Rappresenta un enumeratore del file system per un certo path
    /// </summary>
    internal class FileSystemHierarchicalEnumerable : IHierarchicalEnumerable
    {

        private DirectoryInfo path;
        private FileSystemHierarchyData parent;
        private string viewPath;
        private FileSystemHierarchicalDataSourceView dataSourceView;

        internal FileSystemHierarchicalEnumerable(DirectoryInfo path, FileSystemHierarchyData parent, string viewPath, FileSystemHierarchicalDataSourceView dataSourceView)
        {
            this.path = path;
            this.parent = parent;
            this.viewPath = viewPath;
            this.dataSourceView = dataSourceView;
        }

        #region IHierarchicalEnumerable Members

        IHierarchyData IHierarchicalEnumerable.GetHierarchyData(object enumeratedItem)
        {
            FileSystemInfo fi = (FileSystemInfo)enumeratedItem;
            return new FileSystemHierarchyData(parent, fi, String.Format("{0}/{1}", this.viewPath, fi.Name), this.dataSourceView);
        }

        #endregion

        #region IEnumerable Members

        IEnumerator IEnumerable.GetEnumerator()
        {
            // Restituisco prima le directory
            foreach (DirectoryInfo di in this.path.GetDirectories())
            {
                yield return di;
            }

            // Solo se non  stato richiesto solo le directory
            if (this.dataSourceView.ShowHierarchicalFiles)
            {
                // Restituisco anche i files
                foreach (FileInfo fi in this.path.GetFiles())
                {
                    yield return fi;
                }
            }
        }

        #endregion
    }

}