using System;
using System.Collections.Generic;
using System.Data;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text;

namespace ASPItalia.Books.Web.Chapter11
{

    public class FtpParser
    {
        private readonly Encoding encoding;

        private Regex directoryDetailsRegex = new Regex(@"(?i)(?<date>\d{2}-\d{2}-\d{2}\s{2}\d{2}:\d{2}((AM)|(PM)))
\s{7}
(?<dir>\<dir\>)?
\s*
(?<size>\d+)?
\s
(?<name>[^\r\n]+)", RegexOptions.Compiled | RegexOptions.IgnorePatternWhitespace);

        public FtpParser(Encoding encoding)
        {
            this.encoding = encoding;
        }

        public List<FtpFileInfo> ParseListDirectoryDetails(Stream stream)
        {
            return this.ParseListDirectoryDetails(stream, true);
        }
        
        public List<FtpFileInfo> ParseListDirectoryDetails(Stream stream, bool showFiles)
        {
            List<FtpFileInfo> list = new List<FtpFileInfo>();
            
            using (StreamReader reader = new StreamReader(stream))
            {
                string line;
                // Carico una riga per volta
                while ((line = reader.ReadLine()) != null)
                {
                    // Cerco i campi con una regex
                    Match m = directoryDetailsRegex.Match(line);
                    if (m.Success)
                    {
                        int size = 0;
                        // Provo a parserizzare la dimensione
                        if (m.Groups["size"].Length > 0)
                            Int32.TryParse(m.Groups["size"].Value, out size);
                        
                        // Provo a caricare la data
                        DateTime lastModified;
                        DateTime.TryParse(m.Groups["date"].Value, CultureInfo.CreateSpecificCulture("en-US"), DateTimeStyles.AssumeLocal, out lastModified);
                        
                        // Creo l'oggetto che lo rappresenta
                        FtpFileInfo file = new FtpFileInfo(m.Groups["name"].Value, size, (m.Groups["dir"].Length > 0), lastModified);
                        if (showFiles || file.IsDirectory)
                            list.Add(file);
                    }
                }
            }
            
            return list;
        }
    }

    
    public class FtpFileInfo
    {
        private string _name;
        private int _size;
        private bool _isDirectory;
        private DateTime _lastModified;

        public FtpFileInfo(string name, int size, bool isDirectory, DateTime lastModified)
        {
            _name = name;
            _size = size;
            _isDirectory = isDirectory;
            _lastModified = lastModified;
        }

        public string Name
        {
            get { return _name; }
        }

        public int Size
        {
            get { return _size; }
        }

        public bool IsDirectory
        {
            get { return _isDirectory; }
        }

        public DateTime LastModified
        {
            get { return _lastModified; }
        }
    }
}