using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.ComponentModel;


namespace ASPItalia.Books.Web.Chapter11
{

    public class SizeField : DataControlField
    {

        private string _dataField;
        private PropertyDescriptor fieldDescriptor;

        protected override DataControlField CreateField()
        {
            return new SizeField();
        }

        public override void InitializeCell(DataControlFieldCell cell, DataControlCellType cellType, DataControlRowState rowState, int rowIndex)
        {
            // Solo se non  ne un footer ne un header
            if (cellType == DataControlCellType.DataCell)
            {
                Control control = null;
                // Solo se non sono in modalit di modifica o inserimento
                if (!((rowState & DataControlRowState.Edit) == DataControlRowState.Edit))
                {
                    control = cell;
                }
                else
                {
                    // Creo la textbox di modifica
                    TextBox tb = new TextBox();
                    tb.Columns = 5;
                    tb.Text = "0";
                    cell.Controls.Add(tb);
                    control = tb;
                }

                // Intercetto l'evento DataBinding del controllo
                if (control != null)
                    control.DataBinding += new EventHandler(control_DataBinding);
            }
            else
                base.InitializeCell(cell, cellType, rowState, rowIndex);
        }

        public override void ExtractValuesFromCell(System.Collections.Specialized.IOrderedDictionary dictionary, DataControlFieldCell cell, DataControlRowState rowState, bool includeReadOnly)
        {
            if (((rowState & DataControlRowState.Insert) == DataControlRowState.Normal) || this.InsertVisible)
            {
                // Se la  cella non ha figli allora  in modalit lettura
                // quindi prelevo il suo testo
                object value = null;
                if (cell.Controls.Count == 0)
                    value = cell.Text;
                else if (cell.Controls[0] is TextBox) // Altrimenti  una TextBox e leggo il suo contenuto
                    value = ((TextBox)cell.Controls[0]).Text;

                // Aggiungo nel dictionary per il relativo campo il valore
                string key = this.DataField;
                if (dictionary.Contains(key)) // Se gi lo sostituisco
                    dictionary[key] = value;
                else
                    dictionary.Add(key, value);
            }
        }

        void control_DataBinding(object sender, EventArgs e)
        {
            Control control = (Control)sender;
            Control container = control.NamingContainer;

            object value = this.GetValue(container);
            // Se  in modalit lettura, cambio il contenuto della cella
            if (control is TableCell)
            {
                string text = FormatValue(value);
                ((TableCell)control).Text = text;
            }
            else if (control is TextBox)
            {
                ((TextBox)control).Text = value.ToString();
            }
        }

        // non utilizzato
        void control_DataBinding2(object sender, EventArgs e)
        {
            Control control = (Control)sender;
            Control container = control.NamingContainer;

            object value = DataBinder.Eval(container, this.DataField);
            ((TableCell)control).Text = FormatValue(value);
        }

        private string FormatValue(object value)
        {
            if (value == null || Convert.IsDBNull(value))
                value = 0;
            int size = Convert.ToInt32(value);

            double s = 0d;

            // Trova l'esponente arrotondando per difetto
            int e;
            if (size <= 0)
                e = 0;
            else
                e = (int)Math.Floor(Math.Log(size, 1024));

            // In base all'esponente determino la dimensione
            string type = null;
            switch (e)
            {
                case 1:
                    type = "KB";
                    break;
                case 2:
                    type = "MB";
                    break;
                case 3:
                    type = "GB";
                    break;
                case 4:
                    type = "TB";
                    break;
                default:
                    type = "Bytes";
                    break;
            }

            // Divido per 1024 elvato per l'esemponente
            s = Math.Round(size / Math.Pow(1024, e));
            return String.Format("{0} {1}", s, type);
        }

        private object GetValue(Control container)
        {
            if (this.DesignMode) return 0;

            object dataItem = DataBinder.GetDataItem(container);

            // Cerco i metadati della propriet se non  stato ancora fatto
            // ottimizzo la ricerca via reflection salvando le informazioni
            if (fieldDescriptor == null)
            {
                fieldDescriptor = TypeDescriptor.GetProperties(dataItem).Find(this.DataField, true);
                if (fieldDescriptor == null)
                    throw new HttpException(String.Format("Cannot find property {0}", this.DataField));
            }

            return fieldDescriptor.GetValue(dataItem);
        }

        [TypeConverter("System.Web.UI.Design.DataSourceViewSchemaConverter, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"), Category("Data"), DefaultValue("")]
        public virtual string DataField
        {
            get
            {
                if (this._dataField == null)
                {
                    object o = base.ViewState["DataField"];
                    if (o != null)
                        this._dataField = (string)o;
                    else
                        this._dataField = string.Empty;
                }
                return this._dataField;
            }
            set
            {
                if (!Object.Equals(value, base.ViewState["DataField"]))
                {
                    base.ViewState["DataField"] = value;
                    this._dataField = value;
                    this.OnFieldChanged();
                }
            }
        }
 
    }

}