Imports System
Imports System.Collections
Imports System.Data
Imports System.Configuration
Imports System.Net
Imports System.Web
Imports System.Web.Security
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts
Imports System.Web.UI.HtmlControls
Imports System.Text

Namespace ASPItalia.Books.Web.Chapter11

    Public Class FtpDataSource
        Inherits HierarchicalDataSourceControl
        Implements IDataSource
        Private _viewNames() As String

        Private _host As Uri
        Private _proxy As IWebProxy
        Private _credentials As ICredentials
        Private _usePassive As Boolean
        Private _initialPath As String

        Private _parser As FtpParser = New FtpParser(Encoding.ASCII)

#Region "Public methods"

        Public Function MakeDirectory(ByVal directoryName As String) As Boolean
            Dim request As FtpWebRequest = GetFtpRequest(directoryName)
            request.Method = WebRequestMethods.Ftp.MakeDirectory

            Using response As FtpWebResponse = CType(request.GetResponse(), FtpWebResponse)
                Return (response.StatusCode = FtpStatusCode.PathnameCreated)
            End Using

        End Function

#End Region

        Protected Overrides Function GetHierarchicalView(ByVal viewPath As String) As HierarchicalDataSourceView
            Return New FtpHierarchicalDataSourceView(Me, viewPath)
        End Function

        Friend Function GetFtpRequest(ByVal path As String) As FtpWebRequest
            If Me.Host Is Nothing Then
                Throw New HttpException(String.Format("Please specified a Host for FtpDataSource {0}", Me.ID))
            End If

            Dim uri As UriBuilder = New UriBuilder(Me.Host)
            If Not String.IsNullOrEmpty(Me.InitialPath) Then
                uri.Path = VirtualPathUtility.Combine(uri.Path, Me.InitialPath)
            End If
            If Not String.IsNullOrEmpty(path) Then
                uri.Path += path
            End If

            ' Creo l'oggetto ftp
            Dim ftp As FtpWebRequest = CType(FtpWebRequest.Create(uri.Uri), FtpWebRequest)

            ' Setto le varie impostazioni
            If Not Me.Credentials Is Nothing Then
                ftp.Credentials = Me.Credentials
            End If
            If Not Me.Proxy Is Nothing Then
                ftp.Proxy = Me.Proxy
            End If
            ftp.UsePassive = Me.UsePassive

            Return ftp
        End Function

#Region "Properties"

        '/ <summary>
        '/ Indica se mostrare in modo gerarchico anche i files. Default false
        '/ </summary>
        Public Property ShowHierarchicalFiles() As Boolean
            Get
                Dim o As Object = ViewState("ShowHierarchicalFiles")
                If Not o Is Nothing Then
                    Return CType(o, Boolean)
                Else
                    Return False
                End If
            End Get
            Set(ByVal Value As Boolean)
                ViewState("ShowHierarchicalFiles") = True

                Me.OnDataSourceChanged(EventArgs.Empty)
            End Set
        End Property

        '/ <summary>
        '/ Indica il path iniziale sul FTP
        '/ </summary>
        Public Property InitialPath() As String
            Get
                If _initialPath Is Nothing Then
                    _initialPath = String.Empty
                End If
                Return _initialPath
            End Get
            Set(ByVal Value As String)
                If Me.InitialPath <> Value Then
                    _initialPath = Value
                    Me.OnDataSourceChanged(EventArgs.Empty)
                End If
            End Set
        End Property

        '/ <summary>
        '/ Uri del host ftp
        '/ </summary>
        Public Property Host() As Uri
            Get
                Return _host
            End Get
            Set(ByVal Value As Uri)
                If Me.Host <> Value Then
                    _host = Value
                    Me.OnDataSourceChanged(EventArgs.Empty)
                End If
            End Set
        End Property

        '/ <summary>
        '/ Eventuale proxy da utilizzare
        '/ </summary>
        Public Property Proxy() As IWebProxy
            Get
                Return _proxy
            End Get
            Set(ByVal Value As IWebProxy)
                If Not Me.Proxy Is Value Then
                    _proxy = Value
                    Me.OnDataSourceChanged(EventArgs.Empty)
                End If
            End Set
        End Property

        '/ <summary>
        '/ Credenziali di accesso
        '/ </summary>
        Public Property Credentials() As ICredentials
            Get
                Return _credentials
            End Get
            Set(ByVal Value As ICredentials)
                If Not Me.Credentials Is Value Then
                    _credentials = Value
                    Me.OnDataSourceChanged(EventArgs.Empty)
                End If
            End Set
        End Property

        '/ <summary>
        '/ Modalit passiva
        '/ </summary>
        Public Property UsePassive() As Boolean
            Get
                Return _usePassive
            End Get
            Set(ByVal Value As Boolean)
                If Me.UsePassive <> Value Then
                    _usePassive = Value
                    Me.OnDataSourceChanged(EventArgs.Empty)
                End If
            End Set
        End Property

        Friend ReadOnly Property Parser() As FtpParser
            Get
                Return Me._parser
            End Get
        End Property

#End Region

#Region "IDataSource implement"

        Custom Event DataSourceChanged As EventHandler Implements IDataSource.DataSourceChanged
            AddHandler(ByVal value As EventHandler)
                AddHandler DirectCast(Me, IHierarchicalDataSource).DataSourceChanged, value
            End AddHandler
            RemoveHandler(ByVal value As EventHandler)
                RemoveHandler DirectCast(Me, IHierarchicalDataSource).DataSourceChanged, value
            End RemoveHandler
            RaiseEvent(ByVal sender As Object, ByVal e As EventArgs)

            End RaiseEvent
        End Event

        Private Function GetView(ByVal viewName As String) As DataSourceView Implements IDataSource.GetView
            If viewName.Length = 0 Then
                viewName = "DefaultView"
            End If
            Return New FtpDataSourceView(Me, viewName)
        End Function

        Private Function GetViewNames() As ICollection Implements IDataSource.GetViewNames
            If Me._viewNames Is Nothing Then
                Me._viewNames = New String() {"DefaultView"}
            End If
            Return Me._viewNames
        End Function

#End Region

    End Class

End Namespace

