using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.ComponentModel;

namespace ASPItalia.Books.Web.Chapter13
{

    public class Acronym : WebControl, IPostBackEventHandler
    {

        private static object EventClick = new object();

        private bool clicked;

        public Acronym() : base(HtmlTextWriterTag.Acronym)
        {}

        public bool Clickable
        {
            get
            {
                object o = this.ViewState["Clickable"];
                if (o != null)
                    return (bool)o;
                else
                    return false;
            }
            set
            {
                this.ViewState["Clickable"] = value;
            }
        }

        public string Text
        {
            get
            {
                object o = this.ViewState["Text"];
                if (o != null)
                    return (string)o;
                else
                    return String.Empty;
            }
            set
            {
                this.ViewState["Text"] = value;
            }
        }

        // Il Click2  una versione pi ottimizzata
        public event EventHandler Click;

        [Category("Action"), Description("Evento click sull'acronimo")]
        public event EventHandler Click2
        {
            add
            {
                base.Events.AddHandler(Acronym.EventClick, value);
            }
            remove
            {
                base.Events.RemoveHandler(Acronym.EventClick, value);
            }
        }

        protected virtual void OnClick2(EventArgs e)
        {
            EventHandler handler = (EventHandler)base.Events[Acronym.EventClick];
            if (handler != null)
                handler(this, e);
        }

        protected virtual void OnClick(EventArgs e)
        {
            if (Click != null)
                Click(this, e);
        }

        protected override object SaveControlState()
        {
            object o = base.SaveControlState();
            // Se il control state base  nullo e clicked  false
            //  inutile che salvi qualcosa
            if (o == null && !this.clicked)
                return null;

            return new Pair(o, this.clicked);
        }

        protected override void LoadControlState(object savedState)
        {
            Pair p = savedState as Pair;
            if (p != null)
            {
                // Carico il primo valore nella classe base
                base.LoadControlState(p.First);
                this.clicked = (bool)p.Second;
            }
        }

        protected override void Render(HtmlTextWriter writer)
        {
            // Se non  cliccabile rimuovo gli stili per simulare il link
            if (!this.Clickable)
            {
                this.Style.Remove(HtmlTextWriterStyle.Cursor);
                this.Style.Remove(HtmlTextWriterStyle.TextDecoration);
            }
            else
            {
                // Se  cliccabile metto degli style per simulare il link
                this.Style[HtmlTextWriterStyle.Cursor] = "pointer";
                this.Style[HtmlTextWriterStyle.TextDecoration] = "underline"; ;
            }

            base.Render(writer);
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            writer.Write(this.Text);
        }

        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            base.AddAttributesToRender(writer);

            if (this.Clickable)
                writer.AddAttribute(HtmlTextWriterAttribute.Onclick, this.Page.ClientScript.GetPostBackEventReference(this, String.Empty));
        }

        #region IPostBackEventHandler Members

        void IPostBackEventHandler.RaisePostBackEvent(string eventArgument)
        {
            this.OnClick(EventArgs.Empty);
        }

        #endregion
    }

}
